/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.access;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ProgressMonitor;

import com.carlosbcruz.access.model.Function;
import com.carlosbcruz.util.PropertiesController;

/**
 * Main form of Access application
 *
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccessMainForm extends JFrame {

    JTextArea functionNames = new JTextArea(
        "# Enter SAP function names one in each line...", 30, 40);

    JPanel managerPanel = new JPanel();

    JPanel mainPanel = new JPanel();

    JPanel inputPanel = new JPanel();

    JTextField sapMode = new JTextField();
    JTextField sapClient = new JTextField();
    JTextField groupName = new JTextField();
    JTextField hostName = new JTextField();
    JTextField systemName = new JTextField();

    char[] currentPassword = null;

    JButton updateButton = new JButton("Ok");
    JPasswordField password = new JPasswordField(10);
    JButton generationButton = new JButton("Document functions");

    // Save the properties
    String functions[] = new String[0];

    boolean connectionPoolAlreadyCreated = false;

    // Step to show the time progress
    public final static int QUARTER_OF_SECOND = 250;
    final private JFrame thisComponent = this;
    // Progress bar
    javax.swing.Timer timer;
    ProgressMonitor progressMonitor;
    String progress = new String();
    String exception = null;
    boolean stopTask = false;
    int lengthOfTask = 100;
    int currentTask = 0;
    boolean running = false;

    /**
     *  Version 1 of AccessMainForm
     */
    private static final long serialVersionUID = 1L;

    /**
     * Main constructor
     */
    AccessMainForm() {

        PropertiesController.loadProperties();

        addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent e) {

                saveContext();
                System.exit(1);
            }
        });

        String content = FunctionsRepository.getContent();
        if (content != null) {
            functionNames.setText(content);
        }

        JScrollPane inputScrollPane = new JScrollPane(functionNames,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(inputScrollPane, BorderLayout.CENTER);

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(generationButton);
        mainPanel.add(buttonPanel, BorderLayout.EAST);

        generationButton.addActionListener(new GenerateCommandAction());

        managerPanel.add(inputPanel);

        setContentPane(managerPanel);

        inputPanel.setLayout(new GridLayout(7, 2));

        sapMode.setText(PropertiesController.getStringPropertyWithDefault(
            AccessConstants.SAP_MODE, "0"));
        sapClient.setText(PropertiesController.getStringPropertyWithDefault(
            AccessConstants.SAP_CLIENT, "001"));
        groupName.setText(PropertiesController.getStringPropertyWithDefault(
            AccessConstants.GROUP_NAME, "GROUP0"));
        hostName.setText(PropertiesController.getStringPropertyWithDefault(
            AccessConstants.HOST_NAME, "server.abc.def.com"));
        systemName.setText(PropertiesController.getStringPropertyWithDefault(
            AccessConstants.SYSTEM_NAME, "SYN"));

        inputPanel.add(new JLabel("Mode:"));
        inputPanel.add(sapMode);
        inputPanel.add(new JLabel("Client:"));
        inputPanel.add(sapClient);
        inputPanel.add(new JLabel("Group name:"));
        inputPanel.add(groupName);
        inputPanel.add(new JLabel("Host name:"));
        inputPanel.add(hostName);
        inputPanel.add(new JLabel("System name:"));
        inputPanel.add(systemName);
        inputPanel.add(new JLabel("Password:"));
        inputPanel.add(password);
        inputPanel.add(new JLabel());
        inputPanel.add(updateButton);

        updateButton.addActionListener(new UpdateCommandAction());

        JMenuBar menuBar = new JMenuBar();
        JMenu mainManu = new JMenu(new MainMenuAction());

        JMenuItem inputMenuItem = new JMenuItem(new ConnectionAction());
        JMenuItem executionMenuItem = new JMenuItem(new ExecutionAction());
        JMenuItem exitMenuItem = new JMenuItem(new ExitAction());

        mainManu.add(inputMenuItem);
        mainManu.add(executionMenuItem);
        mainManu.addSeparator();
        mainManu.add(exitMenuItem);

        menuBar.add(mainManu);
        setJMenuBar(menuBar);

        pack();

        // Window width
        String storedWidth = PropertiesController.getStringPropertyWithDefault(
            AccessConstants.WINDOW_WIDTH_KEY, String.valueOf(getWidth()));
        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_WIDTH_KEY, storedWidth);

        // Window height
        String storedHeight = PropertiesController
                .getStringPropertyWithDefault(
                    AccessConstants.WINDOW_HEIGHT_KEY, String
                            .valueOf(getHeight()));
        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_HEIGHT_KEY, storedHeight);

        setSize(Integer.parseInt(storedWidth), Integer.parseInt(storedHeight));

        // Set the size and location of the window
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension screenSize = toolkit.getScreenSize();

        String defaultX = String.valueOf((screenSize.width - getWidth()) / 2);
        String locationX = PropertiesController.getStringPropertyWithDefault(
            AccessConstants.WINDOW_LOCATION_X, defaultX);
        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_LOCATION_X, locationX);

        String defaultY = String.valueOf((screenSize.height - getHeight()) / 2);
        String locationY = PropertiesController.getStringPropertyWithDefault(
            AccessConstants.WINDOW_LOCATION_Y, defaultY);
        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_LOCATION_Y, locationY);

        setLocation(Integer.parseInt(locationX), Integer.parseInt(locationY));
        password.requestFocus();
        setVisible(true);

    }

    /**
     * Action for the main menu
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class MainMenuAction extends AbstractAction {

        /**
         * Version
         */
        private static final long serialVersionUID = 1L;

        MainMenuAction() {

            super();
            putValue(Action.NAME, "Panel selection");
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_P));
        }

        public void actionPerformed(ActionEvent e) {

            // Do nothing
        }

    }

    /**
     * Exit the application
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class ExitAction extends AbstractAction {

        /**
         * Version
         */
        private static final long serialVersionUID = 1L;

        ExitAction() {

            super();
            putValue(Action.NAME, "Exit");
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_X));
        }

        public void actionPerformed(ActionEvent e) {

            saveContext();
            System.exit(1);

        }
    }

    /**
     * Handle the menu selection for the connection
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class ConnectionAction extends AbstractAction {

        /**
         * Version
         */
        private static final long serialVersionUID = 1L;

        ConnectionAction() {

            super();
            putValue(Action.NAME, "Connection panel");
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
        }

        public void actionPerformed(ActionEvent e) {

            managerPanel.remove(mainPanel);
            managerPanel.add(inputPanel);

            thisComponent.validate();
            thisComponent.repaint();
            sapMode.requestFocus();
        }

    }

    /**
     * Handle the menu selection for the execution painel
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class ExecutionAction extends AbstractAction {

        /**
         * version
         */
        private static final long serialVersionUID = 1L;

        ExecutionAction() {

            super();
            putValue(Action.NAME, "Execution panel");
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_E));
        }

        public void actionPerformed(ActionEvent e) {

            managerPanel.remove(inputPanel);
            managerPanel.add(mainPanel);
            thisComponent.validate();
            thisComponent.repaint();
            functionNames.requestFocus();
        }

    }

    /**
     * Execute the update command
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class UpdateCommandAction implements ActionListener {

        /* (non-Javadoc)
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {

            PropertiesController.setStringProperty(AccessConstants.SAP_MODE,
                sapMode.getText());
            PropertiesController.setStringProperty(AccessConstants.SAP_CLIENT,
                sapClient.getText());
            PropertiesController.setStringProperty(AccessConstants.GROUP_NAME,
                groupName.getText());
            PropertiesController.setStringProperty(AccessConstants.HOST_NAME,
                hostName.getText());
            PropertiesController.setStringProperty(AccessConstants.SYSTEM_NAME,
                systemName.getText());
            currentPassword = password.getPassword();
        }
    }

    /**
     * Generate the documentation of functions
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class GenerateCommandAction implements ActionListener, Runnable {

        public void run() {

            try {
                running = true;

                // Save the content
                FunctionsRepository.writeContent(functionNames.getText());
                // Save the properties
                progress = "Function list.";
                String functions[] = FunctionsRepository.getFunctionList();
                currentTask = 1;

                // Inform the length of task
                lengthOfTask = functions.length + 3;

                progress = "Connetion pools.";
                if (!connectionPoolAlreadyCreated) {
                    Access.createConnectionPools(sapMode.getText(), sapClient
                            .getText(), groupName.getText(),
                        hostName.getText(), systemName.getText(), new String(
                            currentPassword));
                    connectionPoolAlreadyCreated = true;
                }
                currentTask = 2;

                for (int i = 0; i < functions.length; i++) {

                    progress = functions[i];

                    // If stop was requested
                    if (stopTask)
                        return;
                    // Inform the current task
                    currentTask = 3 + i;

                    if (functions[i] == null || functions[i].trim().equals(""))
                        continue;

                    if (functions[i] != null && functions[i].length() > 0) {
                        if (functions[i].trim().charAt(0) == '#') {
                            highLightFunction(functions[i].trim(), true);
                            try {
                                Thread.sleep(100);
                            }
                            catch(InterruptedException e) {
                                
                                e.printStackTrace();
                            }
                            continue;
                        }
                    }

                    highLightFunction(functions[i], false);
                    Function function = new Function();
                    function.setName(functions[i]);
                    Access.createFunctionModelFromSAP(function);
                    FunctionsRepository.writeFunctionDocumentation(functions[i]
                        + AccessConstants.FUNCTION_DOCUMENTATION_EXTENTION,
                        function.getComment());
                }

                progress = AccessConstants.FINISHED_PROCESSING;
            }
            catch(Exception e) {
                exception = e.getMessage() + "\n";
            }
        }

        /* (non-Javadoc)
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {

            if (running == true) {
                JOptionPane
                        .showMessageDialog(
                            thisComponent,
                            "There is a documentation process running.\nYou have to wait it to finish or cancel the process.",
                            "Information", JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            if (currentPassword == null) {
                JOptionPane
                        .showMessageDialog(
                            thisComponent,
                            "You have to set the password first. Access the connection painal.",
                            "Information", JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            exception = null;
            stopTask = false;
            progress = "Starting...";
            FunctionsRepository.setOutputDirectoyBasedOnCurrentTime();

            // Prepare the progress bar
            progressMonitor = new ProgressMonitor(thisComponent,
                "Generating SAP Functions calls documentation...", "", 0, 100);
            progressMonitor.setProgress(0);

            // Start the process
            Thread generationThread = new Thread(new GenerateCommandAction());
            generationThread.start();

            // Start the timer to update the progress monitor
            timer = new javax.swing.Timer(QUARTER_OF_SECOND,
                new TimerListener());
            timer.start();
        }
    }

    /**
     * Highlight the function in the input text
     * @param function the function
     */
    private void highLightFunction(String function, boolean flagGreen) {

        String content = functionNames.getText();

        if (function == null) {
            functionNames.setSelectionStart(0);
            functionNames.setSelectionEnd(0);
            return;
        }
        int index = content.indexOf(function);
        if (index != -1) {

            Color oldColor = functionNames.getSelectionColor();

            if (flagGreen) {
                functionNames.setSelectionColor(new Color(255, 100, 100));
            }

            functionNames.setSelectionStart(index);
            functionNames.setSelectionEnd(index + function.length());

            functionNames.setSelectionColor(oldColor);

        }
    }

    /**
     * Print the progress message of files reading
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class TimerListener implements ActionListener {

        public void actionPerformed(ActionEvent evt) {

            // If the user pressed the cancel or the task is finised than stop
            if (progressMonitor.isCanceled()
                || progress.equals(AccessConstants.FINISHED_PROCESSING)
                || exception != null) {

                progressMonitor.close();
                stopTask = true;
                running = false;
                timer.stop();

                if (exception != null) {
                    JOptionPane.showMessageDialog(thisComponent, exception,
                        "An exception has ocurred.", JOptionPane.ERROR_MESSAGE);

                } else {
                    if (progressMonitor.isCanceled()) {
                        JOptionPane.showMessageDialog(thisComponent,
                            "Cancel requested.", "Information",
                            JOptionPane.INFORMATION_MESSAGE);
                    } else {
                        JOptionPane.showMessageDialog(thisComponent,
                            "Finished processing.", "Information",
                            JOptionPane.INFORMATION_MESSAGE);
                    }
                }
                highLightFunction(null, false);

            } else {
                // Present information to the user
                progressMonitor.setNote(progress);
                progressMonitor.setProgress((int)((double)currentTask
                    / (double)lengthOfTask * 100.0));
            }
        }
    }

    /**
     * Save the context 
     */
    private void saveContext() {

        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_WIDTH_KEY, String.valueOf(getWidth()));
        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_HEIGHT_KEY, String.valueOf(getHeight()));
        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_LOCATION_X, String
                    .valueOf((int)getLocation().getX()));
        PropertiesController.setStringProperty(
            AccessConstants.WINDOW_LOCATION_Y, String
                    .valueOf((int)getLocation().getY()));

        // Save the properties
        PropertiesController.saveProperties();

        // Save the content
        FunctionsRepository.writeContent(functionNames.getText());
    }

    /**
     * Main method of Access application
     * @param args
     */
    public static void main(String[] args) {

        AccessMainForm accessMainForm = new AccessMainForm();
    }
}
