/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.access;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;

/**
 * Provide the list of available functions
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class FunctionsRepository {

    private static final String FUNCTION_FILE = "Functions.txt";

    static File directory = null;

    /**
     * Clear the repository 
     */
    public static void nukeRepository() {

        File file = new File(FUNCTION_FILE);
        file.delete();
    }

    /**
     * Write the content
     * @return the content 
     */
    public static String getContent() {

        // Content of the file
        StringBuffer content = new StringBuffer();

        try {

            FileReader fileReader = new FileReader(new File(FUNCTION_FILE));
            BufferedReader bufferedReader = new BufferedReader(fileReader);

            // Current line
            String line = new String();
            while ((line = bufferedReader.readLine()) != null) {

                content.append(line);
                content.append("\n");

            }
        }
        catch(FileNotFoundException e1) {
            return null;
        }
        catch(IOException e1) {

            e1.printStackTrace();
        }

        return content.toString();
    }

    /**
     * Read the functions from file
     * @return the function list as an array of strings
     */
    public static String[] getFunctionList() {

        File functionFile = new File(FUNCTION_FILE);

        String functions[] = null;
        try {
            FileReader fileReader = new FileReader(functionFile);
            BufferedReader reader = new BufferedReader(fileReader);

            String line = reader.readLine();
            ArrayList functionsRead = new ArrayList();
            while (line != null) {
                functionsRead.add(line);
                line = reader.readLine();
            }
            reader.close();
            fileReader.close();
            functions = new String[functionsRead.size()];
            functions = (String[])functionsRead.toArray(functions);
        }
        catch(FileNotFoundException e) {
            e.printStackTrace();
            System.exit(0);
        }
        catch(IOException e) {
            e.printStackTrace();
            System.exit(0);
        }

        return functions;
    }

    /**
     * Save the content to a file
     * @param content
     */
    public static void writeContent(String content) {

        try {

            FileWriter fileWriter = new FileWriter(new File(FUNCTION_FILE));
            BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

            bufferedWriter.write(content);

            bufferedWriter.close();
            fileWriter.close();
        }
        catch(FileNotFoundException e1) {

            e1.printStackTrace();
        }
        catch(IOException e1) {

            e1.printStackTrace();
        }
    }

    /**
     * Set the current output directory based on the current time
     */
    public static void setOutputDirectoyBasedOnCurrentTime() {

        Calendar calendar = Calendar.getInstance();
        directory = new File(String.valueOf(calendar.get(Calendar.YEAR)) + "-"
            + String.valueOf(calendar.get(Calendar.MONTH) + 1) + "-"
            + String.valueOf(calendar.get(Calendar.DAY_OF_MONTH)) + "_"
            + String.valueOf(calendar.get(Calendar.HOUR)) + "_"
            + String.valueOf(calendar.get(Calendar.MINUTE)) + "_"
            + String.valueOf(calendar.get(Calendar.SECOND)));

    }

    /**
     * Save the documentation of a function
     * @param fileName the output file
     * @param content the documentation of the function
     */
    public static void writeFunctionDocumentation(String fileName,
        String content) {

        if (directory == null)
            setOutputDirectoyBasedOnCurrentTime();
        directory.mkdir();

        try {

            FileWriter fileWriter = new FileWriter(new File(directory
                    .getAbsolutePath()
                + "/" + fileName));
            BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

            bufferedWriter.write(content);

            bufferedWriter.close();
            fileWriter.close();
        }
        catch(FileNotFoundException e1) {

            e1.printStackTrace();
        }
        catch(IOException e1) {

            e1.printStackTrace();
        }
    }
}
