/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.access.model;

import java.util.ArrayList;

/**
 * Store a function information
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Function {

	private String name = null;

	private String comment = new String();
	
	private ArrayList simpleParameters = new ArrayList();

	private ArrayList complexParameters = new ArrayList();

	/**
	 * @param i parameter relative position
	 * @return The simple parameter
	 */
	public SimpleParameter getSimpleParameter(int i) {
		return (SimpleParameter) simpleParameters.get(i);
	}

	/**
	 * Add a simple parameter if it does not exist.
	 * @param parameter the simple parameter
	 */
	public void addSimpleParameter(SimpleParameter parameter) {
		
		for (int i = 0; i < simpleParameters.size(); i++) {
			SimpleParameter test = (SimpleParameter) simpleParameters.get(i);
			
			if (test.getName().equals(parameter.getName())) 
				return;
		}
		
		simpleParameters.add(parameter);
	}

	/**
	 * Inform the number of parameters
	 * @return the number of parameters
	 */
	public int getNumberOfSimpleParameters() {
		return simpleParameters.size();
	}

	/**
	 * Retrieve the complex parameter of relative position i
	 * @param i The relative position
	 * @return the complex parameter
	 */
	public ComplexParameter getComplexParameter(int i) {
		return (ComplexParameter) complexParameters.get(i);
	}

	/**
	 * Add a complex parameter
	 * @param parameter the complex parameter
	 */
	public void addComplexParameter(ComplexParameter parameter) {
		complexParameters.add(parameter);
	}

	/**
	 * Inform the number of complex parameters
	 * @return the number of complex parameters
	 */
	public int getNumberOfComplexParameters() {
		return complexParameters.size();
	}

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * @return Returns the comment.
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * @param comment The comment to set.
	 */
	public void addComment(String comment) {
		this.comment += comment + "\r\n";
	}
}
