/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.sapproxy.access;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ProgressMonitor;
import javax.swing.text.Caret;

import com.carlosbcruz.sapproxy.generator.FunctionDocumentation;
import com.carlosbcruz.sapproxy.generator.ProxyGenerator;
import com.carlosbcruz.sapproxy.generator.SystemDestinationDataProvider;
import com.carlosbcruz.sapproxy.generator.model.Function;
import com.carlosbcruz.util.ExceptionSupport;
import com.carlosbcruz.util.FileSupport;
import com.carlosbcruz.util.PasswordUtil;
import com.carlosbcruz.util.SimpleProperties;
import com.carlosbcruz.util.StringSupport;

/**
 * Main form of Access application
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccessMainForm extends JFrame {

	private boolean useEncryptedFile = true;
	String ENCRYPTION_KEY = "49.152.93.186.235.51.97.57.22.150.124.145.213.132.29.161";

	private static final String FUNCTION_FILE = "Functions.txt";
	private static final String PASSWORD_FILE = "Password.txt";
	private static final String NEW_PASSWORD_FILE = "Password.new";

	public static final String WINDOW_WIDTH_KEY = "WINDOW_WIDTH_KEY";
	public static final String WINDOW_HEIGHT_KEY = "WINDOW_HEIGHT_KEY";
	public static final String WINDOW_LOCATION_X = "WINDOW_LOCATION_X";
	public static final String WINDOW_LOCATION_Y = "WINDOW_LOCATION_Y";

	public static final String FUNCTION_DOCUMENTATION_EXTENTION = ".txt";

	public static final String FINISHED_PROCESSING = "Finished processing";

	public static final String SAP_MODE = "SAP_MODE";
	public static final String SAP_CLIENT = "SAP_CLIENT";
	public static final String GROUP_NAME = "GROUP_NAME";
	public static final String HOST_NAME = "HOST_NAME";
	public static final String SYSTEM_NAME = "SYSTEM_NAME";
	public static final String USER_NAME = "USER_NAME";

	private String directory;

	String initialInformation = "# Enter SAP function names one in each line followed by the target package name...\n# Enter \"extends <packageName>\"  for main abstract classes package.";
	JTextArea functionNames = new JTextArea(initialInformation, 30, 40);

	JPanel managerPanel = new JPanel();

	JPanel mainPanel = new JPanel();

	JPanel inputPanel = new JPanel();

	JTextField sapClient = new JTextField();
	JTextField groupName = new JTextField();
	JTextField hostName = new JTextField();
	JTextField systemName = new JTextField();
	JTextField userName = new JTextField();

	char[] currentPassword = null;

	JButton updateButton = new JButton("Ok");
	JPasswordField password = new JPasswordField(10);
	JButton generationButton = new JButton("Document functions");

	JPasswordField newPassword = new JPasswordField(10);

	// Save the properties
	String functions[] = new String[0];

	// Step to show the time progress
	public final static int QUARTER_OF_SECOND = 250;

	final private JFrame thisComponent = this;

	// Progress bar
	javax.swing.Timer timer;
	ProgressMonitor progressMonitor;
	String progress = new String();
	String exception = null;
	boolean stopTask = false;
	int lengthOfTask = 100;
	int currentTask = 0;
	boolean running = false;

	/**
	 * Version 1 of AccessMainForm
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Main constructor
	 */
	AccessMainForm() {

		addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {

				saveContext();
				System.exit(1);
			}
		});

		StringBuffer fileContent = FileSupport.readTextFile(FUNCTION_FILE);

		if (fileContent == null) {
			FileSupport.writeTextFile(FUNCTION_FILE, initialInformation);
			fileContent = new StringBuffer(initialInformation);
		}

		String content = fileContent.toString();
		if (content != null) {
			functionNames.setText(content);
		}

		JScrollPane inputScrollPane = new JScrollPane(functionNames,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(inputScrollPane, BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(generationButton);
		mainPanel.add(buttonPanel, BorderLayout.EAST);

		generationButton.addActionListener(new GenerateCommandAction());

		inputPanel.setLayout(new GridLayout(8, 2));

		sapClient.setText(SimpleProperties.getStringPropertyWithDefault(
				AccessConstants.SAP_CLIENT, "001"));
		groupName.setText(SimpleProperties.getStringPropertyWithDefault(
				AccessConstants.GROUP_NAME, "GROUP0"));
		hostName.setText(SimpleProperties.getStringPropertyWithDefault(
				AccessConstants.HOST_NAME, "server.abc.def.com"));
		systemName.setText(SimpleProperties.getStringPropertyWithDefault(
				AccessConstants.SYSTEM_NAME, "SYN"));
		userName.setText(SimpleProperties.getStringPropertyWithDefault(
				AccessConstants.USER_NAME, "User"));

		inputPanel.add(new JLabel("Client:"));
		inputPanel.add(sapClient);
		inputPanel.add(new JLabel("Group name:"));
		inputPanel.add(groupName);
		inputPanel.add(new JLabel("Host name:"));
		inputPanel.add(hostName);
		inputPanel.add(new JLabel("System name:"));
		inputPanel.add(systemName);
		inputPanel.add(new JLabel("User name:"));
		inputPanel.add(userName);
		inputPanel.add(new JLabel("Password:"));

		if (useEncryptedFile) {

			StringBuffer encryptedPassword = FileSupport
					.readTextFile(PASSWORD_FILE);
			if (encryptedPassword == null) {
				String message = "Password not found. This instance is configure to read the password from file: "
						+ PASSWORD_FILE;
				ExceptionSupport.handleException(message);
				inputPanel.add(password);
				useEncryptedFile = false;
			} else {
				String decriptedPassword;

				String encryptedPasswordFiltered = StringSupport.filterString(
						encryptedPassword.toString(), new char[] { '0', '1',
								'2', '3', '4', '5', '6', '7', '8', '9', '.' });
				try {
					decriptedPassword = PasswordUtil.decryptPassword(
							ENCRYPTION_KEY, encryptedPasswordFiltered);
				} catch (Exception e) {
					ExceptionSupport.handleException(e);
					throw new RuntimeException(e);
				}
				currentPassword = decriptedPassword.toCharArray();
				inputPanel.add(new JLabel("Password found on file."));
			}
		} else {
			inputPanel.add(password);
		}
		inputPanel.add(new JLabel("Attention:"));
		inputPanel.add(new JLabel("Using wrong PW may block the ID."));
		inputPanel.add(new JLabel());
		inputPanel.add(updateButton);

		managerPanel.setLayout(new BorderLayout());
		managerPanel.add(inputPanel, BorderLayout.CENTER);

		JPanel newPasswordPanel = new JPanel();
		newPasswordPanel.setBorder(BorderFactory
				.createCompoundBorder(BorderFactory
						.createTitledBorder("Generate new password file."),
						BorderFactory.createEmptyBorder(5, 5, 5, 5)));

		newPasswordPanel.add(new JLabel("Password:"));

		newPasswordPanel.add(newPassword);
		JButton generateButton = new JButton("Generate");
		newPasswordPanel.add(generateButton);
		generateButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent event) {
				char newPasswordChar[] = newPassword.getPassword();
				String newPasswordString = new String(newPasswordChar);
				try {
					String encryptedPassword = PasswordUtil.encryptPassword(
							ENCRYPTION_KEY, newPasswordString);
					FileSupport.writeTextFile(NEW_PASSWORD_FILE,
							encryptedPassword);
					JOptionPane.showMessageDialog(null, "The file: "
							+ NEW_PASSWORD_FILE
							+ " was generated.\nYou can rename to "
							+ PASSWORD_FILE + " to take effect.");
				} catch (Exception exception) {
					ExceptionSupport.handleException(exception);
					throw new RuntimeException(exception);
				}

			}

		});

		managerPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createTitledBorder("Enter SAP connection information."),
				BorderFactory.createEmptyBorder(5, 5, 5, 5)));

		managerPanel.add(newPasswordPanel, BorderLayout.SOUTH);

		setContentPane(managerPanel);

		this.setTitle("SAP Proxy Generator");

		updateButton.addActionListener(new UpdateCommandAction());

		pack();

		// Set the size and location of the window
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolkit.getScreenSize();

		String defaultX = String.valueOf((screenSize.width - getWidth()) / 2);
		String locationX = SimpleProperties.getStringPropertyWithDefault(
				AccessConstants.WINDOW_LOCATION_X, defaultX);
		SimpleProperties.setStringProperty(AccessConstants.WINDOW_LOCATION_X,
				locationX);

		String defaultY = String.valueOf((screenSize.height - getHeight()) / 2);
		String locationY = SimpleProperties.getStringPropertyWithDefault(
				AccessConstants.WINDOW_LOCATION_Y, defaultY);
		SimpleProperties.setStringProperty(AccessConstants.WINDOW_LOCATION_Y,
				locationY);

		setLocation(Integer.parseInt(locationX), Integer.parseInt(locationY));
		password.requestFocus();
		setVisible(true);

	}

	/**
	 * Execute the update command
	 *  
	 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
	 */
	class UpdateCommandAction implements ActionListener {

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent
		 * )
		 */
		public void actionPerformed(ActionEvent e) {

			SimpleProperties.setStringProperty(AccessConstants.SAP_CLIENT,
					sapClient.getText());
			SimpleProperties.setStringProperty(AccessConstants.GROUP_NAME,
					groupName.getText());
			SimpleProperties.setStringProperty(AccessConstants.HOST_NAME,
					hostName.getText());
			SimpleProperties.setStringProperty(AccessConstants.SYSTEM_NAME,
					systemName.getText());
			SimpleProperties.setStringProperty(AccessConstants.USER_NAME,
					userName.getText());

			if (!useEncryptedFile) {
				currentPassword = password.getPassword();
			}

			SystemDestinationDataProvider.setHost(hostName.getText());
			SystemDestinationDataProvider.setSystemName(systemName.getText());
			SystemDestinationDataProvider.setClient(sapClient.getText());
			SystemDestinationDataProvider.setGroup(groupName.getText());
			SystemDestinationDataProvider.setUser(userName.getText());
			SystemDestinationDataProvider.setPassword(String
					.valueOf(currentPassword));

			SystemDestinationDataProvider.selfRegister();

			managerPanel.remove(inputPanel);
			managerPanel.add(mainPanel, BorderLayout.CENTER);
			thisComponent.pack();
			thisComponent.validate();
			thisComponent.repaint();
			functionNames.requestFocus();

		}
	}

	/**
	 * Generate the documentation of functions
	 *  
	 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
	 */
	class GenerateCommandAction implements ActionListener, Runnable {

		public void run() {

			try {
				running = true;

				// Save the content
				FileSupport.writeTextFile(FUNCTION_FILE, functionNames
						.getText());
				// Save the properties
				progress = "Function list.";
				String functions[] = FileSupport.readAllLines(FUNCTION_FILE);
				currentTask = 1;

				// Inform the length of task
				lengthOfTask = functions.length + 3;

				progress = "Connetion pools.";
				currentTask = 2;

				directory = FileSupport.getCurrentDateForFileName();

				FileSupport.createDirectory(directory);

				String packageExtends = "com.default";
				for (int i = 0; i < functions.length; i++) {

					progress = functions[i];

					// If stop was requested
					if (stopTask)
						return;
					// Inform the current task
					currentTask = 3 + i;

					if (functions[i] == null || functions[i].trim().equals(""))
						continue;

					if (functions[i] != null && functions[i].length() > 0) {
						if (functions[i].trim().charAt(0) == '#') {
							highLightFunction(functions[i].trim());
							try {
								Thread.sleep(100);
							} catch (InterruptedException e) {

								e.printStackTrace();
							}
							continue;
						}
					}

					int space = functions[i].indexOf(' ');
					String functionName = functions[i];
					String packageName = "com.default";
					if (space > 0) {
						functionName = functions[i].substring(0, space).trim();
						packageName = functions[i].substring(space,
								functions[i].length()).trim();
					}
					highLightFunction(functions[i]);

					if ("extends".equals(functionName)) {

						ProxyGenerator.generateMainPortType(directory,
								packageName);

						packageExtends = packageName;

						continue;
					}

					Function function = new Function();
					function.setName(functionName);
					function.setPackageName(packageName);

					function = FunctionDocumentation
							.createFunctionModelFromSAP(function);

					ProxyGenerator.generateProxy(directory, packageExtends,
							function);

					FileSupport.writeTextFile(directory, functions[i]
							+ AccessConstants.FUNCTION_DOCUMENTATION_EXTENTION,
							function.getComment());

				}

				progress = AccessConstants.FINISHED_PROCESSING;

			} catch (Exception e) {
				exception = e.getMessage() + "\n";
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent
		 * )
		 */
		public void actionPerformed(ActionEvent e) {

			if (running == true) {
				JOptionPane
						.showMessageDialog(
								thisComponent,
								"There is a documentation process running.\nYou have to wait it to finish or cancel the process.",
								"Information", JOptionPane.INFORMATION_MESSAGE);
				return;
			}

			if (currentPassword == null) {
				JOptionPane
						.showMessageDialog(
								thisComponent,
								"You have to set the password first. Access the connection painal.",
								"Information", JOptionPane.INFORMATION_MESSAGE);
				return;
			}

			exception = null;
			stopTask = false;
			progress = "Starting...";
			directory = FileSupport.getCurrentDateForFileName();

			// Prepare the progress bar
			progressMonitor = new ProgressMonitor(thisComponent,
					"Generating SAP Functions calls documentation...", "", 0,
					100);
			progressMonitor.setProgress(0);

			// Start the process
			Thread generationThread = new Thread(new GenerateCommandAction());
			generationThread.start();

			// Start the timer to update the progress monitor
			timer = new javax.swing.Timer(QUARTER_OF_SECOND,
					new TimerListener());
			timer.start();
		}
	}

	/**
	 * Highlight the function in the input text
	 * 
	 * @param function
	 *            the function
	 */
	private void highLightFunction(String function) {

		String content = functionNames.getText();

		if (function == null) {
			functionNames.setSelectionStart(0);
			functionNames.setSelectionEnd(0);
			return;
		}
		int index = content.indexOf(function);
		if (index != -1) {

			Caret caret = functionNames.getCaret();

			caret.setSelectionVisible(true);
			functionNames.setSelectionStart(0);
			functionNames.setSelectionEnd(index + function.length());
		}
	}

	/**
	 * Print the progress message of files reading
	 *  
	 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
	 */
	class TimerListener implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			// If the user pressed the cancel or the task is finised than stop
			if (progressMonitor.isCanceled()
					|| progress.equals(AccessConstants.FINISHED_PROCESSING)
					|| exception != null) {

				progressMonitor.close();
				stopTask = true;
				running = false;
				timer.stop();

				if (exception != null) {
					JOptionPane.showMessageDialog(thisComponent, exception,
							"An exception has ocurred.",
							JOptionPane.ERROR_MESSAGE);

				} else {
					if (progressMonitor.isCanceled()) {
						JOptionPane.showMessageDialog(thisComponent,
								"Cancel requested.", "Information",
								JOptionPane.INFORMATION_MESSAGE);
					} else {
						JOptionPane.showMessageDialog(thisComponent,
								"Finished processing.", "Information",
								JOptionPane.INFORMATION_MESSAGE);
					}
				}
				highLightFunction(null);

			} else {
				// Present information to the user
				progressMonitor.setNote(progress);
				progressMonitor.setProgress((int) ((double) currentTask
						/ (double) lengthOfTask * 100.0));
			}
		}
	}

	/**
	 * Save the context
	 */
	private void saveContext() {

		SimpleProperties.setStringProperty(AccessConstants.WINDOW_WIDTH_KEY,
				String.valueOf(getWidth()));
		SimpleProperties.setStringProperty(AccessConstants.WINDOW_HEIGHT_KEY,
				String.valueOf(getHeight()));
		SimpleProperties.setStringProperty(AccessConstants.WINDOW_LOCATION_X,
				String.valueOf((int) getLocation().getX()));
		SimpleProperties.setStringProperty(AccessConstants.WINDOW_LOCATION_Y,
				String.valueOf((int) getLocation().getY()));

		// Save the content
		FileSupport.writeTextFile(FUNCTION_FILE, functionNames.getText());
	}

	/**
	 * Main method of Access application
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		new AccessMainForm();
	}
}
