/**
 * Maintenance:
 * 
 * 2010-Jul-08      Carlos F.Bella Cruz     Creation.
 */
package com.carlosbcruz.sapproxy.generator.model;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * Store a function information
 */
public class Function implements Serializable {

	/**
	 * Serial
	 */
	private static final long serialVersionUID = 1L;

	private String name = new String();

	private String comment = new String();

	private ArrayList<SimpleParameter> inputParameters = new ArrayList<SimpleParameter>();
	private ArrayList<ComplexParameter> inputStructures = new ArrayList<ComplexParameter>();

	private ArrayList<SimpleParameter> outputParameters = new ArrayList<SimpleParameter>();

	private ArrayList<ComplexParameter> complexParameters = new ArrayList<ComplexParameter>();

	private String packageName = new String();

	/**
	 * Retrieve the input parameters.
	 * 
	 * @return The input parameters.
	 */
	public ArrayList<SimpleParameter> getInputParameters() {
		return inputParameters;
	}

	/**
	 * Set the input parameters.
	 * 
	 * @param inputParameters
	 *            The input parameters.
	 */
	public void setInputParameters(ArrayList<SimpleParameter> inputParameters) {
		this.inputParameters = inputParameters;
	}

	/**
	 * Set the output parameters.
	 * 
	 * @return The output parameters.
	 */
	public ArrayList<SimpleParameter> getOutputParameters() {
		return outputParameters;
	}

	/**
	 * Set the input structures.
	 * @return The input structures.
	 */
	public ArrayList<ComplexParameter> getInputStructures() {
		return inputStructures;
	}

	/**
	 * Get the input structures.
	 * @param inputStructures The input structures.
	 */
	public void setInputStructures(ArrayList<ComplexParameter> inputStructures) {
		this.inputStructures = inputStructures;
	}

	/**
	 * Retrieve the output parameters.
	 * 
	 * @param outputParameters
	 *            The output parameters.
	 */
	public void setOutputParameters(ArrayList<SimpleParameter> outputParameters) {
		this.outputParameters = outputParameters;
	}

	/**
	 * Inform the number of parameters
	 * 
	 * @return the number of input parameters
	 */
	public int getNumberOfInputParameters() {
		return inputParameters.size();
	}

	/**
	 * Inform the number of parameters
	 * 
	 * @return the number of output parameters
	 */
	public int getNumberOfOutputParameters() {
		return outputParameters.size();
	}

	/**
	 * Retrieve the complex parameter of relative position i
	 * 
	 * @param i
	 *            The relative position
	 * @return the complex parameter
	 */
	public ComplexParameter getComplexParameter(int i) {
		return (ComplexParameter) complexParameters.get(i);
	}

	/**
	 * Add a complex parameter
	 * 
	 * @param parameter
	 *            the complex parameter
	 */
	public void addComplexParameter(ComplexParameter parameter) {
		complexParameters.add(parameter);
	}

	/**
	 * Inform the number of complex parameters
	 * 
	 * @return the number of complex parameters
	 */
	public int getNumberOfComplexParameters() {
		return complexParameters.size();
	}

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return Returns the comment.
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * @param comment
	 *            The comment to set.
	 */
	public void addComment(String comment) {
		this.comment += comment + "\r\n";
	}

	/**
	 * @return The package name.
	 */
	public String getPackageName() {
		return packageName;
	}

	/**
	 * @param packageName
	 *            The package name.
	 */
	public void setPackageName(String packageName) {
		this.packageName = packageName;
	}

}
